import React, { createContext, useContext, useState, useCallback, useMemo } from "react";
import { MathUtils } from "three";

export type LogType = "log" | "info" | "warning" | "error" | "success";

export interface LogEntry {
  id: string;
  type: LogType;
  message: string;
  timestamp: Date;
}

interface LoggerContextValue {
  logs: LogEntry[];
  setLogs: React.Dispatch<React.SetStateAction<LogEntry[]>>;
  isLogListVisible: boolean;
  setIsLogListVisible: React.Dispatch<React.SetStateAction<boolean>>;
  log: (message: string) => void;
  info: (message: string) => void;
  warn: (message: string) => void;
  error: (message: string) => void;
  success: (message: string) => void;
  clear: () => void;
}

const LoggerContext = createContext<LoggerContextValue | undefined>(undefined);

export const LoggerProvider: React.FC<{ children: React.ReactNode }> = ({
  children,
}) => {
  const [logs, setLogs] = useState<LogEntry[]>([]);
  const [isLogListVisible, setIsLogListVisible] = useState<boolean>(false);

  const addLog = useCallback(
    (type: LogType, message: string) => {
      const newLog: LogEntry = {
        id: MathUtils.generateUUID(),
        type,
        message,
        timestamp: new Date(),
      };
      setLogs((prevLogs) => [...prevLogs, newLog]);
    },
    []
  );

  const loggerMethods: LoggerContextValue = useMemo(
    () => ({
      logs,
      setLogs,
      isLogListVisible,
      setIsLogListVisible,
      log: (message: string) => addLog("log", message),
      info: (message: string) => addLog("info", message),
      warn: (message: string) => addLog("warning", message),
      error: (message: string) => addLog("error", message),
      success: (message: string) => addLog("success", message),
      clear: () => setLogs([]),
    }),
    [logs, setLogs, isLogListVisible, setIsLogListVisible, addLog]
  );

  return (
    <LoggerContext.Provider value={loggerMethods}>
      {children}
    </LoggerContext.Provider>
  );
};

export const useLogger = () => {
  const context = useContext(LoggerContext);
  if (!context) {
    throw new Error("useLogger must be used within a LoggerProvider");
  }
  return context;
};
