import { useCallback, useEffect, useRef } from "react";
import * as THREE from 'three';
import { useFrame } from "@react-three/fiber";
import { useMaterialStore } from "../../../../../store/simulation/useMaterialStore";
import { useProductStore } from "../../../../../store/simulation/useProductStore";
import { useSelectedProduct } from "../../../../../store/simulation/useSimulationStore";
import { usePlayButtonStore, useAnimationPlaySpeed, usePauseButtonStore, useResetButtonStore } from "../../../../../store/usePlayButtonStore";

interface SpawnInstance {
    lastSpawnTime: number | null;
    startTime: number;
    spawnCount: number;
    params: {
        material: string;
        intervalMs: number;
        totalCount: number;
        action: ConveyorAction;
    };
    pauseStartTime: number;
    remainingTime: number;
    isPaused: boolean;
}

export function useSpawnHandler() {
    const { addMaterial } = useMaterialStore();
    const { getModelUuidByActionUuid, getPointUuidByActionUuid } = useProductStore();
    const { isPlaying } = usePlayButtonStore();
    const { isPaused } = usePauseButtonStore();
    const { speed } = useAnimationPlaySpeed();
    const { isReset } = useResetButtonStore();
    const { selectedProduct } = useSelectedProduct();

    const activeSpawns = useRef<Map<string, SpawnInstance>>(new Map());

    const clearAllSpawns = useCallback(() => {
        activeSpawns.current.clear();
    }, []);

    useEffect(() => {
        if (isReset) {
            clearAllSpawns();
        }
    }, [isReset, clearAllSpawns]);

    const spawnLogStatus = (materialUuid: string, status: string) => {
        // console.log(`${materialUuid}, ${status}`);
    }

    const createNewMaterial = useCallback((materialType: string, action: ConveyorAction) => {
        const modelUuid = getModelUuidByActionUuid(selectedProduct.productId, action.actionUuid);
        const pointUuid = getPointUuidByActionUuid(selectedProduct.productId, action.actionUuid);
        if (!modelUuid || !pointUuid) return;

        const newMaterial: MaterialSchema = {
            materialId: THREE.MathUtils.generateUUID(),
            materialName: `${materialType}-${Date.now()}`,
            materialType: materialType,
            isActive: false,
            isVisible: true,
            isRendered: true,
            current: {
                modelUuid: modelUuid,
                pointUuid: pointUuid,
                actionUuid: action?.actionUuid || ''
            },
            weight: 1,
            cost: 1
        };

        if (action.triggers[0].triggeredAsset?.triggeredModel.modelUuid &&
            action.triggers[0].triggeredAsset?.triggeredPoint?.pointUuid &&
            action.triggers[0].triggeredAsset?.triggeredAction?.actionUuid
        ) {
            newMaterial.next = {
                modelUuid: action.triggers[0].triggeredAsset?.triggeredModel.modelUuid,
                pointUuid: action.triggers[0].triggeredAsset?.triggeredPoint?.pointUuid,
                actionUuid: action.triggers[0].triggeredAsset?.triggeredAction?.actionUuid
            }
        }

        addMaterial(newMaterial);
        return newMaterial;
    }, [addMaterial, getModelUuidByActionUuid, getPointUuidByActionUuid, selectedProduct.productId]);

    useEffect(() => {
        const currentTime = performance.now();

        activeSpawns.current.forEach((spawn) => {
            if (isPaused && !spawn.isPaused) {
                if (spawn.lastSpawnTime === null) {
                    spawn.remainingTime = Math.max(0, spawn.params.intervalMs - (currentTime - spawn.startTime));
                } else {
                    spawn.remainingTime = Math.max(0, spawn.params.intervalMs - (currentTime - spawn.lastSpawnTime));
                }
                spawn.pauseStartTime = currentTime;
                spawn.isPaused = true;
            } else if (!isPaused && spawn.isPaused) {
                if (spawn.remainingTime > 0) {
                    if (spawn.lastSpawnTime === null) {
                        spawn.startTime = currentTime - (spawn.params.intervalMs - spawn.remainingTime);
                    } else {
                        spawn.lastSpawnTime = currentTime - (spawn.params.intervalMs - spawn.remainingTime);
                    }
                }
                spawn.isPaused = false;
                spawn.pauseStartTime = 0;
                spawn.remainingTime = 0;
            }
        });
    }, [isPaused]);

    useFrame(() => {
        if (!isPlaying || isPaused || isReset) return;

        const currentTime = performance.now();
        const completedActions: string[] = [];

        activeSpawns.current.forEach((spawn, actionUuid) => {
            const { material, intervalMs, totalCount, action } = spawn.params;
            const isFirstSpawn = spawn.lastSpawnTime === null;

            // First spawn
            if (isFirstSpawn) {
                const elapsed = currentTime - spawn.startTime;
                if (elapsed >= intervalMs) {
                    const createdMaterial = createNewMaterial(material, action);
                    if (createdMaterial) {
                        spawnLogStatus(createdMaterial.materialId, `[${elapsed.toFixed(2)}ms] Spawned ${material} (1/${totalCount})`);
                    }
                    spawn.lastSpawnTime = currentTime;
                    spawn.spawnCount = 1;

                    if (totalCount <= 1) {
                        completedActions.push(actionUuid);
                    }
                }
                return;
            }

            // Subsequent spawns
            if (spawn.lastSpawnTime !== null) {
                const timeSinceLast = currentTime - spawn.lastSpawnTime;
                if (timeSinceLast >= intervalMs) {
                    const count = spawn.spawnCount + 1;
                    const createdMaterial = createNewMaterial(material, action);
                    if (createdMaterial) {
                        spawnLogStatus(createdMaterial.materialId, `[${timeSinceLast.toFixed(2)}ms] Spawned ${material} (${count}/${totalCount})`);
                    }
                    spawn.lastSpawnTime = currentTime;
                    spawn.spawnCount = count;

                    if (count >= totalCount) {
                        completedActions.push(actionUuid);
                    }
                }
            }
        });

        completedActions.forEach(actionUuid => {
            activeSpawns.current.delete(actionUuid);
        });
    });

    const handleSpawn = useCallback((action: ConveyorAction) => {
        if (!action || action.actionType !== 'spawn') return;

        const { material, spawnInterval = 0, spawnCount = 1, actionUuid } = action;
        const intervalMs = spawnInterval * 1000;

        if (activeSpawns.current.has(actionUuid)) {
            activeSpawns.current.delete(actionUuid);
        }

        activeSpawns.current.set(actionUuid, {
            lastSpawnTime: null,
            startTime: performance.now(),
            spawnCount: 0,
            params: {
                material,
                intervalMs,
                totalCount: spawnCount,
                action: action
            },
            pauseStartTime: 0,
            remainingTime: 0,
            isPaused: false
        });
    }, []);

    useEffect(() => {
        return () => {
            clearAllSpawns();
        };
    }, [clearAllSpawns]);

    return {
        handleSpawn,
        clearCurrentSpawn: clearAllSpawns
    };
}