import React, { useEffect, useState, useRef } from 'react';
import * as THREE from 'three';
import { useFrame, useThree } from '@react-three/fiber';
import { usePauseButtonStore, usePlayButtonStore } from '../../../../../store/usePlayButtonStore';

interface MaterialAnimatorProps {
    matRef: React.RefObject<THREE.Mesh>;
    material: MaterialSchema;
    currentSpeed: number;
    onAnimationComplete?: () => void;
}

function MaterialAnimator({
    matRef,
    material,
    currentSpeed,
    onAnimationComplete
}: MaterialAnimatorProps) {
    const { scene } = useThree();
    const [targetPosition, setTargetPosition] = useState<THREE.Vector3 | null>(null);
    const [isAnimating, setIsAnimating] = useState(false);
    const animationState = useRef({
        startTime: 0,
        startPosition: new THREE.Vector3(),
        totalDistance: 0,
        pausedTime: 0,
        isPaused: false,
        lastFrameTime: 0
    });

    const { isPlaying } = usePlayButtonStore();
    const { isPaused } = usePauseButtonStore();

    const getWorldPosition = (uuid: string): THREE.Vector3 | null => {
        const obj = scene.getObjectByProperty('uuid', uuid);
        if (!obj) return null;
        const position = new THREE.Vector3();
        obj.getWorldPosition(position);
        return position;
    };

    // Handle target position changes and play state
    useEffect(() => {
        if (!isPlaying || !material.next?.pointUuid) {
            setIsAnimating(false);
            return;
        }

        const newTarget = getWorldPosition(material.next.pointUuid);
        if (newTarget && matRef.current) {
            animationState.current.startPosition.copy(matRef.current.position);
            animationState.current.totalDistance = animationState.current.startPosition.distanceTo(newTarget);
            animationState.current.startTime = performance.now() - animationState.current.pausedTime;
            animationState.current.pausedTime = 0;
            animationState.current.isPaused = false;
            setTargetPosition(newTarget);
            setIsAnimating(true);
        }
    }, [material.next?.pointUuid, isPlaying]);

    // Handle pause/unpause
    useEffect(() => {
        if (isPaused) {
            animationState.current.isPaused = true;
            setIsAnimating(false);
            // Record the time when paused
            animationState.current.pausedTime = performance.now() - animationState.current.startTime;
        } else {
            animationState.current.isPaused = false;
            if (isPlaying && targetPosition && !isAnimating) {
                // Resume from where we left off
                animationState.current.startTime = performance.now() - animationState.current.pausedTime;
                setIsAnimating(true);
            }
        }
    }, [isPaused]);

    useFrame(() => {
        if (!matRef.current || !targetPosition || !isAnimating || animationState.current.isPaused || !isPlaying) {
            return;
        }

        const currentTime = performance.now();
        // Calculate elapsed time since animation start, minus any paused time
        const elapsed = (currentTime - animationState.current.startTime) / 1000;
        const progress = Math.min(1, (currentSpeed * elapsed) / animationState.current.totalDistance);

        matRef.current.position.lerpVectors(
            animationState.current.startPosition,
            targetPosition,
            progress
        );

        if (progress >= 1) {
            matRef.current.position.copy(targetPosition);
            setIsAnimating(false);
            onAnimationComplete?.();
        }
    });

    return null;
}

export default React.memo(MaterialAnimator);